clear; close all; clc;
addpath(genpath('.\utils'))

%
% DEPcode (2018):
% This code implements a version of the descriptor described in the paper:
%
% Discrete time Evolution Process Descriptor for shape analysis and matching
% ACM Transactions on Graphics, Vol. 37, No. 1, Article 4. January 2018
%
% Simone Melzi - University of Verona
% Maks Ovsjanikov - LIX cole Polytechnique 
% Giorgio Roffo - University of Glasgow
% Marco Cristani - University of Verona
% Umberto Castellani - University of Verona
%
% obtain a visualization of the DEP descriptor as the one shown in 
% figure 3 in the paper
%
% original parameters;
% c = 0.1; flag_dist = 'Dijkstra'; n_scale = 100; f0 = ones(Src.nv,1);
% Src.n_points = Src.nv
%
% If you set flag_dist = 'SpectroMeter' it is 10 times faster.
% If you set rc.n_points < Src.nv the computation is faster.
%

%% load shape
[X,T] = readOff('.\data\kid');
Src = MeshInfo(X, T, 200);

%% select parameters
% DEP parameters
n_scale = 16; % 100; % 6;
c = 0.1; % 0.5; % 0.9;
f0 = ones(Src.nv,1);
flag_dist = 'SpectroMeter'; % 'Dijkstra'; % SpectroMeter is 10 times faster
Src.n_points = Src.nv; % 3000;
if n_scale == 16
    deltas = [0.01,0.02,0.025,0.03,0.04 0.05,0.07,0.075,0.08, 0.09,0.1,...
              0.15, 0.35, 0.55, 0.8, 1];
elseif n_scale == 100
    deltas = [0.005:0.005:0.375,0.4:0.025:1];
elseif n_scale == 6
    deltas100 = [0.005:0.005:0.375,0.4:0.025:1];
    deltas_idxs = [1,20:20:100]; %[1 20 40 60 80 100];
    deltas = deltas100(deltas_idxs);
end

%%
Src = all_in_DEP(Src,c,deltas,flag_dist,Src.n_points);

%%
figure
if n_scale == 16
    for q = 1:n_scale
        subplot(2,8,q)
        trisurf(Src.T,Src.X(:,1),Src.X(:,2),Src.X(:,3),Src.DEP(:,q),...
                'SpecularStrength',0.15);
        axis equal; axis off, shading interp; view([0 0]); colormap(jet)
    end
elseif n_scale == 100
    for q = 1:10
        subplot(2,5,q)
        trisurf(Src.T,Src.X(:,1),Src.X(:,2),Src.X(:,3),Src.DEP(:,1+10*(q-1)),...
                'SpecularStrength',0.15);
        axis equal; axis off, shading interp; view([0 0]); colormap(jet)       
    end
elseif n_scale == 6
    for q = 1:n_scale
        subplot(1,6,q)
        trisurf(Src.T,Src.X(:,1),Src.X(:,2),Src.X(:,3),Src.DEP(:,q),...
                'SpecularStrength',0.15);
        axis equal; axis off, shading interp; view([0 0]); colormap(jet)       
    end
end
