function [F_lb2] = compute_map_desc(Src,Tar,Src_desc,Tar_desc,numEigsSrc)

numEigsTar = numEigsSrc;
SrcLaplaceBasis = Src.laplaceBasis; SrcEigenvalues = Src.eigenvalues;
TarLaplaceBasis = Tar.laplaceBasis; TarEigenvalues = Tar.eigenvalues;
Src.laplaceBasis = SrcLaplaceBasis(:,1:numEigsSrc); Src.eigenvalues = SrcEigenvalues(1:numEigsSrc);
Tar.laplaceBasis = TarLaplaceBasis(:,1:numEigsTar); Tar.eigenvalues = TarEigenvalues(1:numEigsTar);

%% Descriptors
fct_src = Src_desc;
fprintf('Computing the descriptors...\n');tic;

fct_tar = Tar_desc;

% Subsample descriptors (for faster computation). More descriptors is
% usually better, but can be slower. 
% fct_src = fct_src(:,1:10:end);
% fct_tar = fct_tar(:,1:10:end);

fprintf('done computing descriptors (%d on source and %d on target)\n',size(fct_src,2),size(fct_tar,2)); toc;

assert(size(fct_src,2)==size(fct_tar,2));

% Normalization
no = sqrt(diag(fct_src'*Src.Ae*fct_src))';
fct_src = fct_src ./ repmat(no, [Src.nv,1]);
fct_tar = fct_tar ./ repmat(no, [Tar.nv,1]);

fprintf('Pre-computing the multiplication operators...');tic;
%% Multiplication Operators
numFct = size(fct_src,2);
OpSrc = cell(numFct,1);
OpTar = cell(numFct,1);
for i = 1:numFct
    OpSrc{i} = Src.laplaceBasis'*Src.Ae*(repmat(fct_src(:,i), [1,numEigsSrc]).*Src.laplaceBasis);
    OpTar{i} = Tar.laplaceBasis'*Tar.Ae*(repmat(fct_tar(:,i), [1,numEigsTar]).*Tar.laplaceBasis);
end

Fct_src = Src.laplaceBasis'*Src.Ae*fct_src;
Fct_tar = Tar.laplaceBasis'*Tar.Ae*fct_tar;
fprintf('done\n');toc;

%% Fmap Computation
fprintf('Optimizing the functional map...\n');tic;
Dlb = (repmat(Src.eigenvalues, [1,numEigsTar]) - repmat(Tar.eigenvalues', [numEigsSrc,1])).^2;
Dlb = Dlb/norm(Dlb, 'fro')^2;
constFct = sign(Src.laplaceBasis(1,1)*Tar.laplaceBasis(1,1))*[sqrt(sum(Tar.area)/sum(Src.area)); zeros(numEigsTar-1,1)];

a = 1e-1; % Descriptors preservation
b = 1;    % Commutativity with descriptors
c = 1e-3; % Commutativity with Laplacian 
funObj = @(F) deal( a*sum(sum((reshape(F, [numEigsTar,numEigsSrc])*Fct_src - Fct_tar).^2))/2 + b*sum(cell2mat(cellfun(@(X,Y) sum(sum((X*reshape(F, [numEigsTar,numEigsSrc]) - reshape(F, [numEigsTar,numEigsSrc])*Y).^2)), OpTar', OpSrc', 'UniformOutput', false)), 2)/2 + c*sum( (F.^2 .* Dlb(:))/2 ),...
            a*vec((reshape(F, [numEigsTar,numEigsSrc])*Fct_src - Fct_tar)*Fct_src') + b*sum(cell2mat(cellfun(@(X,Y) vec(X'*(X*reshape(F, [numEigsTar,numEigsSrc]) - reshape(F, [numEigsTar,numEigsSrc])*Y) - (X*reshape(F, [numEigsTar,numEigsSrc]) - reshape(F, [numEigsTar,numEigsSrc])*Y)*Y'), OpTar', OpSrc', 'UniformOutput', false)), 2) + c*F.*Dlb(:));
funProj = @(F) [constFct; F(numEigsTar+1:end)];

F_lb = zeros(numEigsTar*numEigsSrc, 1); F_lb(1) = constFct(1);

% Compute the optional functional map using a quasi-Newton method.
options.verbose = 1;
F_lb = reshape(minConf_PQN(funObj, F_lb, funProj, options), [numEigsTar,numEigsSrc]);
fprintf('done fmap optimization.\n');toc;

%%
fprintf('ICP refinement...');tic;
[F_lb2, ~] = icp_refine(Src.laplaceBasis, Tar.laplaceBasis, F_lb, 5);
fprintf('done\n');toc;
