function Fmaps  = computeFMap(basis1, F, S, basis2, G, R, options)
% Computes a functional map between using basis1 and basis2, the
% descriptors F and G, and the operators in S and R 
% Note that some mapping methods ignore the operators 

Fmaps = [];

t = tic;

alpha = 1;

% Remember: 
% CF = G <--> F'C' = G', so we solve for C as ...
if isempty(S)
    C = (F'\G')';
% If we have operator constraints, need to flatten constraint matrices
% Solving min ||CF-G||_2^2 + sum_i lambda_i*||R_i C - C S_i||_2^2 + 
% The lambdas are computed using the inf norm of the repsective constraint
% matrices
else
    [Md,bd] = descriptorConstraintsRowMajor(F,G);
    if options.norm
        Md = Md/norm(Md,inf);
        bd = bd/norm(Md,inf); 
    end
    M = Md; b = bd;
    for i=1:length(S)
        % Diagonal operators, easier construction
        if isDiag(S{i}) && isDiag(R{i})
            [Mo,bo] = operatorConstraintsDiagonal(S{i},R{i}, colMajor);
        else
            [Mo,bo] = operatorConstraintsRowMajor(S{i},R{i});
        end
        if options.norm
            Mo = alpha*Mo/norm(Mo,inf);
            bo = alpha*bo/norm(Mo,inf); 
        end
        M = [M; Mo];
        b = [b; bo];
    end
    
    % Faster than \ for some reason
    [cs,Rs] = qr(M,b);
    v = Rs\cs;
    
    resid = norm(M*v-b);
    resid_d = norm(Md*v - bd);
    resid_o = norm(Mo*v - bo);
    [mr,nr] = size(R{1});
    [ms,ns] = size(S{1});
    mc = nr; nc = ms;
    if mr ~= mc || nc ~= ns
        error('?');
    end
    C = full(reshape(v',nc,mc)');    
end
Fmaps.maps = C;
fprintf(1, '\t took: %g seconds\n', toc(t));
